# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2017-11-16 01:12
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('pinax_stripe', '0009_auto_20170825_1841'),
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('stripe_id', models.CharField(max_length=191, unique=True)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('business_name', models.TextField(blank=True, null=True)),
                ('business_url', models.TextField(blank=True, null=True)),
                ('charges_enabled', models.BooleanField(default=False)),
                ('country', models.CharField(max_length=2)),
                ('debit_negative_balances', models.BooleanField(default=False)),
                ('decline_charge_on_avs_failure', models.BooleanField(default=False)),
                ('decline_charge_on_cvc_failure', models.BooleanField(default=False)),
                ('default_currency', models.CharField(max_length=3)),
                ('details_submitted', models.BooleanField(default=False)),
                ('display_name', models.TextField()),
                ('email', models.TextField(blank=True, null=True)),
                ('legal_entity_address_city', models.TextField(blank=True, null=True)),
                ('legal_entity_address_country', models.TextField(blank=True, null=True)),
                ('legal_entity_address_line1', models.TextField(blank=True, null=True)),
                ('legal_entity_address_line2', models.TextField(blank=True, null=True)),
                ('legal_entity_address_postal_code', models.TextField(blank=True, null=True)),
                ('legal_entity_address_state', models.TextField(blank=True, null=True)),
                ('legal_entity_dob', models.DateField(null=True)),
                ('legal_entity_first_name', models.TextField(blank=True, null=True)),
                ('legal_entity_gender', models.TextField(blank=True, null=True)),
                ('legal_entity_last_name', models.TextField(blank=True, null=True)),
                ('legal_entity_maiden_name', models.TextField(blank=True, null=True)),
                ('legal_entity_personal_id_number_provided', models.BooleanField(default=False)),
                ('legal_entity_phone_number', models.TextField(blank=True, null=True)),
                ('legal_entity_ssn_last_4_provided', models.BooleanField(default=False)),
                ('legal_entity_type', models.TextField(blank=True, null=True)),
                ('legal_entity_verification_details', models.TextField(blank=True, null=True)),
                ('legal_entity_verification_details_code', models.TextField(blank=True, null=True)),
                ('legal_entity_verification_document', models.TextField(blank=True, null=True)),
                ('legal_entity_verification_status', models.TextField(blank=True, null=True)),
                ('type', models.TextField(blank=True, null=True)),
                ('metadata', jsonfield.fields.JSONField(blank=True, null=True)),
                ('stripe_publishable_key', models.CharField(blank=True, max_length=100, null=True)),
                ('product_description', models.TextField(blank=True, null=True)),
                ('statement_descriptor', models.TextField(blank=True, null=True)),
                ('support_email', models.TextField(blank=True, null=True)),
                ('support_phone', models.TextField(blank=True, null=True)),
                ('timezone', models.TextField(blank=True, null=True)),
                ('tos_acceptance_date', models.DateField(null=True)),
                ('tos_acceptance_ip', models.TextField(blank=True, null=True)),
                ('tos_acceptance_user_agent', models.TextField(blank=True, null=True)),
                ('transfer_schedule_delay_days', models.PositiveSmallIntegerField(null=True)),
                ('transfer_schedule_interval', models.TextField(blank=True, null=True)),
                ('transfer_schedule_monthly_anchor', models.PositiveSmallIntegerField(null=True)),
                ('transfer_schedule_weekly_anchor', models.TextField(blank=True, null=True)),
                ('transfer_statement_descriptor', models.TextField(blank=True, null=True)),
                ('transfers_enabled', models.BooleanField(default=False)),
                ('verification_disabled_reason', models.TextField(blank=True, null=True)),
                ('verification_due_by', models.DateTimeField(blank=True, null=True)),
                ('verification_timestamp', models.DateTimeField(blank=True, null=True)),
                ('verification_fields_needed', jsonfield.fields.JSONField(blank=True, null=True)),
                ('authorized', models.BooleanField(default=True)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='stripe_accounts', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BankAccount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('stripe_id', models.CharField(max_length=191, unique=True)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('account_holder_name', models.TextField()),
                ('account_holder_type', models.TextField()),
                ('bank_name', models.TextField(blank=True, null=True)),
                ('country', models.TextField()),
                ('currency', models.TextField()),
                ('default_for_currency', models.BooleanField(default=False)),
                ('fingerprint', models.TextField()),
                ('last4', models.CharField(max_length=4)),
                ('metadata', jsonfield.fields.JSONField(blank=True, null=True)),
                ('routing_number', models.TextField()),
                ('status', models.TextField()),
                ('account', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='bank_accounts', to='pinax_stripe.Account')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='UserAccount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('account', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_accounts', related_query_name='user_account', to='pinax_stripe.Account')),
            ],
        ),
        migrations.AddField(
            model_name='charge',
            name='available',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='charge',
            name='available_on',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='fee',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='fee_currency',
            field=models.CharField(blank=True, max_length=10, null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='transfer_group',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='amount_reversed',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='application_fee',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='created',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='destination',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='destination_payment',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='failure_code',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='failure_message',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='livemode',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='transfer',
            name='metadata',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='method',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='reversed',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='transfer',
            name='source_transaction',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='source_type',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='statement_descriptor',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='transfer_group',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='type',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='coupon',
            name='metadata',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='customer',
            name='account_balance',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AlterField(
            model_name='customer',
            name='user',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='event',
            name='validated_message',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='plan',
            name='metadata',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='useraccount',
            name='customer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_accounts', related_query_name='user_account', to='pinax_stripe.Customer'),
        ),
        migrations.AddField(
            model_name='useraccount',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_accounts', related_query_name='user_account', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='customer',
            name='stripe_account',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Account'),
        ),
        migrations.AddField(
            model_name='customer',
            name='users',
            field=models.ManyToManyField(related_name='customers', related_query_name='customers', through='pinax_stripe.UserAccount', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='event',
            name='stripe_account',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Account'),
        ),
        migrations.AddField(
            model_name='plan',
            name='stripe_account',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Account'),
        ),
        migrations.AddField(
            model_name='transfer',
            name='stripe_account',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Account'),
        ),
        migrations.AlterUniqueTogether(
            name='useraccount',
            unique_together=set([('user', 'account')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2017-01-08 18:02
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pinax_stripe', '0006_coupon'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charge',
            name='customer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='charges', to='pinax_stripe.Customer'),
        ),
    ]



from django.conf.urls import url
from django.contrib import admin

from ..urls import urlpatterns


class FakeViewForUrl(object):
    def __call__(self):
        raise Exception("Should not get called.")


urlpatterns += [
    url(r"^admin/", admin.site.urls),
    url(r"^the/app/$", FakeViewForUrl, name="the_app"),
    url(r"^accounts/signup/$", FakeViewForUrl, name="signup"),
    url(r"^password/reset/confirm/(?P<token>.+)/$", FakeViewForUrl,
        name="password_reset"),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-11-24 16:30
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pinax_stripe', '0011_auto_20171123_2016'),
    ]

    operations = [
        migrations.AlterField(
            model_name='account',
            name='legal_entity_dob',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='tos_acceptance_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='charge',
            name='amount',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AlterField(
            model_name='charge',
            name='amount_refunded',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AlterField(
            model_name='charge',
            name='customer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='charges', to='pinax_stripe.Customer'),
        ),
        migrations.AlterField(
            model_name='charge',
            name='invoice',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='charges', to='pinax_stripe.Invoice'),
        ),
        migrations.AlterField(
            model_name='charge',
            name='source',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='coupon',
            name='amount_off',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AlterField(
            model_name='coupon',
            name='duration_in_months',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='coupon',
            name='max_redemptions',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='coupon',
            name='percent_off',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='coupon',
            name='redeem_by',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='coupon',
            name='times_redeemed',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='customer',
            name='date_purged',
            field=models.DateTimeField(blank=True, editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='event',
            name='customer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Customer'),
        ),
        migrations.AlterField(
            model_name='eventprocessingexception',
            name='event',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Event'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='attempt_count',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='charge',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoices', to='pinax_stripe.Charge'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='subscription',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Subscription'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='tax',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='tax_percent',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=9, null=True),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='webhooks_delivered_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='plan',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Plan'),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='quantity',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='subscription',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='pinax_stripe.Subscription'),
        ),
        migrations.AlterField(
            model_name='plan',
            name='trial_period_days',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='application_fee_percent',
            field=models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=3, null=True),
        ),
    ]



import stripe

from .. import models, utils


def sync_plans():
    """
    Synchronizes all plans from the Stripe API
    """
    try:
        plans = stripe.Plan.auto_paging_iter()
    except AttributeError:
        plans = iter(stripe.Plan.all().data)

    for plan in plans:
        sync_plan(plan)


def sync_plan(plan, event=None):
    """
    Synchronizes a plan from the Stripe API

    Args:
        plan: data from Stripe API representing a plan
        event: the event associated with the plan
    """

    defaults = {
        "amount": utils.convert_amount_for_db(plan["amount"], plan["currency"]),
        "currency": plan["currency"] or "",
        "interval": plan["interval"],
        "interval_count": plan["interval_count"],
        "name": plan["name"],
        "statement_descriptor": plan["statement_descriptor"] or "",
        "trial_period_days": plan["trial_period_days"],
        "metadata": plan["metadata"]
    }

    obj, created = models.Plan.objects.get_or_create(
        stripe_id=plan["id"],
        defaults=defaults
    )
    utils.update_with_defaults(obj, defaults, created)



import logging

from django.utils import timezone
from django.utils.encoding import smart_str

import stripe

from . import invoices, sources, subscriptions
from .. import hooks, models, utils
from ..conf import settings

logger = logging.getLogger(__name__)


def can_charge(customer):
    """
    Can the given customer create a charge

    Args:
        customer: a pinax.stripe.models.Customer object
    """
    if customer.date_purged is not None:
        return False
    if customer.default_source:
        return True
    return False


def _create_without_account(user, card=None, plan=settings.PINAX_STRIPE_DEFAULT_PLAN, charge_immediately=True, quantity=None):
    cus = models.Customer.objects.filter(user=user).first()
    if cus is not None:
        try:
            stripe.Customer.retrieve(cus.stripe_id)
            return cus
        except stripe.error.InvalidRequestError:
            pass

    # At this point we maybe have a local Customer but no stripe customer
    # let's create one and make the binding
    trial_end = hooks.hookset.trial_period(user, plan)
    stripe_customer = stripe.Customer.create(
        email=user.email,
        source=card,
        plan=plan,
        quantity=quantity,
        trial_end=trial_end
    )
    cus, created = models.Customer.objects.get_or_create(
        user=user,
        defaults={
            "stripe_id": stripe_customer["id"]
        }
    )
    if not created:
        cus.stripe_id = stripe_customer["id"]  # sync_customer will call cus.save()
    sync_customer(cus, stripe_customer)
    if plan and charge_immediately:
        invoices.create_and_pay(cus)
    return cus


def _create_with_account(user, stripe_account, card=None, plan=settings.PINAX_STRIPE_DEFAULT_PLAN, charge_immediately=True, quantity=None):
    cus = user.customers.filter(user_account__account=stripe_account).first()
    if cus is not None:
        try:
            stripe.Customer.retrieve(cus.stripe_id, stripe_account=stripe_account.stripe_id)
            return cus
        except stripe.error.InvalidRequestError:
            pass

    # At this point we maybe have a local Customer but no stripe customer
    # let's create one and make the binding
    trial_end = hooks.hookset.trial_period(user, plan)
    stripe_customer = stripe.Customer.create(
        email=user.email,
        source=card,
        plan=plan,
        quantity=quantity,
        trial_end=trial_end,
        stripe_account=stripe_account.stripe_id,
    )

    if cus is None:
        cus = models.Customer.objects.create(stripe_id=stripe_customer["id"], stripe_account=stripe_account)
        models.UserAccount.objects.create(user=user, account=stripe_account, customer=cus)
    else:
        logger.debug("Update local customer %s with new remote customer %s for user %s, and account %s",
                     cus.stripe_id, stripe_customer["id"], user, stripe_account)
        cus.stripe_id = stripe_customer["id"]  # sync_customer() will call cus.save()
    sync_customer(cus, stripe_customer)
    if plan and charge_immediately:
        invoices.create_and_pay(cus)
    return cus


def create(user, card=None, plan=settings.PINAX_STRIPE_DEFAULT_PLAN, charge_immediately=True, quantity=None, stripe_account=None):
    """
    Creates a Stripe customer.

    If a customer already exists, the existing customer will be returned.

    Args:
        user: a user object
        card: optionally, the token for a new card
        plan: a plan to subscribe the user to
        charge_immediately: whether or not the user should be immediately
                            charged for the subscription
        quantity: the quantity (multiplier) of the subscription
        stripe_account: An account object. If given, the Customer and User relation will be established for you through the UserAccount model.
        Because a single User might have several Customers, one per Account.

    Returns:
        the pinax.stripe.models.Customer object that was created
    """
    if stripe_account is None:
        return _create_without_account(user, card=card, plan=plan, charge_immediately=charge_immediately, quantity=quantity)
    return _create_with_account(user, stripe_account, card=card, plan=plan, charge_immediately=charge_immediately, quantity=quantity)


def get_customer_for_user(user, stripe_account=None):
    """
    Get a customer object for a given user

    Args:
         user: a user object
         stripe_account: An Account object

    Returns:
        a pinax.stripe.models.Customer object
    """
    if stripe_account is None:
        return models.Customer.objects.filter(user=user).first()
    return user.customers.filter(user_account__account=stripe_account).first()


def purge_local(customer):
    customer.user_accounts.all().delete()
    customer.user = None
    customer.date_purged = timezone.now()
    customer.save()


def purge(customer):
    """
    Deletes the Stripe customer data and purges the linking of the transaction
    data to the Django user.

    Args:
        customer: the pinax.stripe.models.Customer object to purge
    """
    try:
        customer.stripe_customer.delete()
    except stripe.InvalidRequestError as e:
        if "no such customer:" not in smart_str(e).lower():
            # The exception was thrown because the customer was already
            # deleted on the stripe side, ignore the exception
            raise
    purge_local(customer)


def link_customer(event):
    """
    Links a customer referenced in a webhook event message to the event object

    Args:
        event: the pinax.stripe.models.Event object to link
    """
    cus_id = None
    customer_crud_events = [
        "customer.created",
        "customer.updated",
        "customer.deleted"
    ]
    event_data_object = event.message["data"]["object"]
    if event.kind in customer_crud_events:
        cus_id = event_data_object["id"]
    else:
        cus_id = event_data_object.get("customer", None)

    if cus_id is not None:
        customer, created = models.Customer.objects.get_or_create(
            stripe_id=cus_id,
            stripe_account=event.stripe_account,
        )
        if event.kind in customer_crud_events:
            sync_customer(customer, event_data_object)

        event.customer = customer
        event.save()


def set_default_source(customer, source):
    """
    Sets the default payment source for a customer

    Args:
        customer: a Customer object
        source: the Stripe ID of the payment source
    """
    stripe_customer = customer.stripe_customer
    stripe_customer.default_source = source
    cu = stripe_customer.save()
    sync_customer(customer, cu=cu)


def sync_customer(customer, cu=None):
    """
    Synchronizes a local Customer object with details from the Stripe API

    Args:
        customer: a Customer object
        cu: optionally, data from the Stripe API representing the customer
    """
    if customer.date_purged is not None:
        return

    if cu is None:
        cu = customer.stripe_customer

    if cu.get("deleted", False):
        purge_local(customer)
        return

    customer.account_balance = utils.convert_amount_for_db(cu["account_balance"], cu["currency"])
    customer.currency = cu["currency"] or ""
    customer.delinquent = cu["delinquent"]
    customer.default_source = cu["default_source"] or ""
    customer.save()
    for source in cu["sources"]["data"]:
        sources.sync_payment_source_from_stripe_data(customer, source)
    for subscription in cu["subscriptions"]["data"]:
        subscriptions.sync_subscription_from_stripe_data(customer, subscription)



from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand

from stripe.error import InvalidRequestError

from ...actions import charges, customers, invoices


class Command(BaseCommand):

    help = "Sync customer data"

    def handle(self, *args, **options):
        User = get_user_model()
        qs = User.objects.exclude(customer__isnull=True)
        count = 0
        total = qs.count()
        for user in qs:
            count += 1
            perc = int(round(100 * (float(count) / float(total))))
            username = getattr(user, user.USERNAME_FIELD)
            self.stdout.write(u"[{0}/{1} {2}%] Syncing {3} [{4}]\n".format(
                count, total, perc, username, user.pk
            ))
            customer = customers.get_customer_for_user(user)
            try:
                customers.sync_customer(customer)
            except InvalidRequestError as exc:
                if exc.http_status == 404:  # pragma: no branch
                    # This user doesn't exist (might be in test mode)
                    continue
                raise exc

            if customer.date_purged is None:
                invoices.sync_invoices_for_customer(customer)
                charges.sync_charges_for_customer(customer)
